@@ -8,7 +8,7 @@ options(width=60)
 
 
 ###################################################
-### code chunk number 2: refGenome.Rnw:51-54
+### code chunk number 2: refGenome.Rnw:55-58
 ###################################################
 library(refGenome)
 beg<-ensemblGenome()
@@ -16,7 +16,7 @@ basedir(beg)<-system.file("extdata",package="refGenome")
 
 
 ###################################################
-### code chunk number 3: refGenome.Rnw:83-86
+### code chunk number 3: refGenome.Rnw:87-90
 ###################################################
 ens_gtf<-"hs.ensembl.62.small.gtf"
 read.gtf(beg,ens_gtf)
@@ -24,14 +24,14 @@ beg
 
 
 ###################################################
-### code chunk number 4: refGenome.Rnw:94-96
+### code chunk number 4: refGenome.Rnw:98-100
 ###################################################
 tableAttributeTypes(beg)
 moveAttributes(beg,c("gene_name","transcript_name","exon_number"))
 
 
 ###################################################
-### code chunk number 5: refGenome.Rnw:127-133 (eval = FALSE)
+### code chunk number 5: refGenome.Rnw:131-137 (eval = FALSE)
 ###################################################
 ## uc<-ucscGenome()
 ## basedir(uc)<-"/my/ucsc/basedir"
@@ -42,7 +42,7 @@ moveAttributes(beg,c("gene_name","transcript_name","exon_number"))
 
 
 ###################################################
-### code chunk number 6: refGenome.Rnw:140-144
+### code chunk number 6: refGenome.Rnw:144-148
 ###################################################
 ucfile<-system.file("extdata", "hs.ucsc.small.RData", package="refGenome")
 uc<-loadGenome(ucfile)
@@ -51,27 +51,27 @@ ens<-loadGenome(ensfile)
 
 
 ###################################################
-### code chunk number 7: refGenome.Rnw:157-158
+### code chunk number 7: refGenome.Rnw:161-162
 ###################################################
 tableSeqids(ens)
 
 
 ###################################################
-### code chunk number 8: refGenome.Rnw:163-165
+### code chunk number 8: refGenome.Rnw:167-169
 ###################################################
 en1<-extractSeqids(ens,"^1$")
 en1
 
 
 ###################################################
-### code chunk number 9: refGenome.Rnw:174-176
+### code chunk number 9: refGenome.Rnw:178-180
 ###################################################
 ensPrimAssembly()
 ucPrimAssembly()
 
 
 ###################################################
-### code chunk number 10: refGenome.Rnw:180-184
+### code chunk number 10: refGenome.Rnw:184-188
 ###################################################
 enpa<-extractSeqids(ens,ensPrimAssembly())
 tableSeqids(enpa)
@@ -80,7 +80,7 @@ tableSeqids(ucpa)
 
 
 ###################################################
-### code chunk number 11: refGenome.Rnw:190-193
+### code chunk number 11: refGenome.Rnw:194-197
 ###################################################
 tableFeatures(enpa)
 enpf<-extractFeature(enpa,"exon")
@@ -88,44 +88,50 @@ enpf
 
 
 ###################################################
-### code chunk number 12: refGenome.Rnw:202-204
+### code chunk number 12: refGenome.Rnw:206-208
 ###################################################
 dxe<-extractByGeneName(enpa,"DDX11L1")
 dxu<-extractByGeneName(ucpa,"DDX11L1")
 
 
 ###################################################
-### code chunk number 13: refGenome.Rnw:211-213
+### code chunk number 13: refGenome.Rnw:213-215
 ###################################################
 tableTranscript.id(enpa)
 tableTranscript.id(ucpa)
 
 
 ###################################################
-### code chunk number 14: refGenome.Rnw:217-219
+### code chunk number 14: refGenome.Rnw:219-221
 ###################################################
 extractTranscript(ens,"ENST00000456328")
 extractTranscript(uc,"uc010nxr.1")
 
 
 ###################################################
-### code chunk number 15: refGenome.Rnw:228-232
+### code chunk number 15: refGenome.Rnw:230-234
 ###################################################
 gpe<-getGenePositions(ens)
 gpe
-gpu<-getGenePositions(ucpa)
+gpu<-getGenePositions(uc)
 gpu
 
 
 ###################################################
-### code chunk number 16: refGenome.Rnw:244-246
+### code chunk number 16: refGenome.Rnw:246-248
 ###################################################
 enex<-refExons(ens)
 ucex<-refExons(uc)
 
 
 ###################################################
-### code chunk number 17: refGenome.Rnw:256-260
+### code chunk number 17: refGenome.Rnw:251-252
+###################################################
+enex
+
+
+###################################################
+### code chunk number 18: refGenome.Rnw:262-266
 ###################################################
 jens<-getSpliceTable(ens)
 jens
@@ -134,16 +140,18 @@ juc
 
 
 ###################################################
-### code chunk number 18: refGenome.Rnw:267-271
+### code chunk number 19: refGenome.Rnw:273-279
 ###################################################
 ujens<-unifyJuncs(jens)
 ujuc<-unifyJuncs(juc)
 jeg<-getGenePositions(jens)
 jug<-getGenePositions(juc)
+head(ujens)
+head(jug)
 
 
 ###################################################
-### code chunk number 19: refGenome.Rnw:285-294
+### code chunk number 20: refGenome.Rnw:293-302
 ###################################################
 qry<-data.frame(
                   id=1:6,
